-- 以下代码使用SQLite 3.18.0 测试通过

-- 创建部门表
CREATE TABLE [dept](
    [dept_id] int, 
    [dept_name] nvarchar(20), 
    [parent_id] int);

-- 创建员工表
CREATE TABLE [employees](
    [emp_id] int, 
    [emp_name] nvarchar(20), 
    [is_manager] bit, 
    [dept_id] int);

-- 插入部门测试数据
insert into dept values(0, '集团', 0);
insert into dept values(1, '西北', 0);
insert into dept values(2, '西安', 1);
insert into dept values(3, '兰州', 1);
insert into dept values(4, '东北', 0);
insert into dept values(5, '哈尔滨', 4);
insert into dept values(6, '吉林', 4);

-- 插入员工测试数据
insert into employees values(1, '西北-经理', 1, 1);
insert into employees values(2, '西北-助理', 0, 1);
insert into employees values(3, '西北-员工', 0, 1);
insert into employees values(4, '西安-经理', 1, 2);
insert into employees values(5, '西安-助理', 0, 2);
insert into employees values(6, '西安-员工', 0, 2);
insert into employees values(7, '兰州-经理', 1, 3);
insert into employees values(8, '兰州-助理', 0, 3);
insert into employees values(9, '兰州-员工', 0, 3);
insert into employees values(10, '东北-经理', 1, 4);
insert into employees values(11, '东北-助理', 0, 4);
insert into employees values(12, '东北-员工', 0, 4);
insert into employees values(13, '哈尔滨-经理', 1, 5);
insert into employees values(14, '哈尔滨-助理', 0, 5);
insert into employees values(15, '哈尔滨-员工', 0, 5);
insert into employees values(16, '吉林-经理', 1, 6);
insert into employees values(17, '吉林-助理', 0, 6);
insert into employees values(18, '吉林-员工', 0, 6);

-- 利用 CTE 查询所有西北地区的员工
WITH
    [depts]([dept_id]) AS(
        SELECT [d].[dept_id]
        FROM   [dept] [d]
               JOIN [employees] [e] ON [d].[dept_id] = [e].[dept_id]
        WHERE  [e].[emp_name] = '西北-经理'
        UNION ALL
        SELECT [d].[dept_id]
        FROM   [dept] [d]
               JOIN [depts] [s] ON [d].[parent_id] = [s].[dept_id]
    )
SELECT *
FROM   [employees]
WHERE  [dept_id] IN (SELECT [dept_id]
       FROM   [depts]);

-- 曼德勃罗集合（Mandelbrot set）
WITH RECURSIVE
  xaxis(x) AS (VALUES(-2.0) UNION ALL SELECT x+0.05 FROM xaxis WHERE x<1.2),
  yaxis(y) AS (VALUES(-1.0) UNION ALL SELECT y+0.1 FROM yaxis WHERE y<1.0),
  m(iter, cx, cy, x, y) AS (
    SELECT 0, x, y, 0.0, 0.0 FROM xaxis, yaxis
    UNION ALL
    SELECT iter+1, cx, cy, x*x-y*y + cx, 2.0*x*y + cy FROM m 
     WHERE (x*x + y*y) < 4.0 AND iter<28
  ),
  m2(iter, cx, cy) AS (
    SELECT max(iter), cx, cy FROM m GROUP BY cx, cy
  ),
  a(t) AS (
    SELECT group_concat( substr(' .+*#', 1+min(iter/7,4), 1), '') 
    FROM m2 GROUP BY cy
  )
SELECT group_concat(rtrim(t),x'0a') FROM a;

-- 数独问题（Sudoku）
WITH RECURSIVE
  input(sud) AS (
    VALUES('53..7....6..195....98....6.8...6...34..8.3..17...2...6.6....28....419..5....8..79')
  ),
  digits(z, lp) AS (
    VALUES('1', 1)
    UNION ALL SELECT
    CAST(lp+1 AS TEXT), lp+1 FROM digits WHERE lp<9
  ),
  x(s, ind) AS (
    SELECT sud, instr(sud, '.') FROM input
    UNION ALL
    SELECT
      substr(s, 1, ind-1) || z || substr(s, ind+1),
      instr( substr(s, 1, ind-1) || z || substr(s, ind+1), '.' )
     FROM x, digits AS z
    WHERE ind>0
      AND NOT EXISTS (
            SELECT 1
              FROM digits AS lp
             WHERE z.z = substr(s, ((ind-1)/9)*9 + lp, 1)
                OR z.z = substr(s, ((ind-1)%9) + (lp-1)*9 + 1, 1)
                OR z.z = substr(s, (((ind-1)/3) % 3) * 3
                        + ((ind-1)/27) * 27 + lp
                        + ((lp-1) / 3) * 6, 1)
         )
  )
SELECT s FROM x WHERE ind=0;
